function PCAscores = PCAcalc(X,classNum,signalname)
% PCAcalc performes PCA and returns two figures from the analysis. 
%   PCAcalc(X,classNum,classLet,signalname) uses the function pca from the 
%   Statistics and Machine Learning Toolbox, and returns the the principal 
%   component (PC) column vectors that together explain at least 95 % of 
%   the variance. Rows of X correspond to observations and columns to 
%   variables. classNum is a N by 1 vector with numbers assigning the 
%   classes data in matrix X belongs to. signalname is a string explaining 
%   the content in matrix X and used in the title of the figures. Figure 1 
%   shows a bar plot of the variance explained in percentage by the PCs in 
%   descending order. Figure 2 shows a scatter plot of the first two PCs 
%   (containing most variance) for the four different classes of data in 
%   the N by P data matrix X. A scatter for each of the k means from PCs of
%   each class is also displayed, using the function kmeans from the 
%   aforementioned toolbox. If the first two PCs dont explain at least 90% 
%   of the variance, then a third PC is added for a 3-D plot
%
    
%   Created by Mathias Falk, last modified 30/06/16


    % Class labels type char
    classLet = char(classNum+'A'-1);

    
    % Perform PCA
    [~,score,~,~,explainedVar] = pca(X);
    
    
    % Extract PCs
    p=0;
    c=0;
 
    for i=1:length(explainedVar)
        if p>95
            break
        end
        p = p + explainedVar(i);
        c = c + 1;
    end
    
    PCAscores = score(:,1:c);
    
    
    % Plot explained variance
    figure
    bar(explainedVar)
     str = sprintf(['%0.2f%% variance explained by first %i PCs ' ...
          'of %s'],p,c,signalname);
    title(str)
    xlabel('PC [number]'); ylabel('Variace [%]')
    set(gca,'FontSize',14)
    

    % Retain first two principal components and scatter plot
    dataPC = score(:,1:2);

    figure
    gscatter(dataPC(:,1),dataPC(:,2),classLet)
    str = sprintf('PCA on %s',signalname);
    title(str)
    xlabel('PC1'); ylabel('PC2')
    set(gca,'FontSize',14)

    % find kmean for values of each class    
    hold on
    for k=1:numel(unique(classNum))
        [~,centroid] = kmeans(dataPC((classNum==k),:),1);
        scatter(centroid(1),centroid(2))
        c = classLet(classNum==k);
        text(centroid(1),centroid(2),c(1),'FontSize',14);
    end
    hold off
    
    
    % 3D plot
    if explainedVar(1)+explainedVar(2) < 90
    % shows that PC3 dont contribute with much variance, and that if rotate
    %  3d plot to a 2d looks same as other 2d plots have made

    dataPC = score(:,1:3);

    figure
    gscatter3(dataPC(:,1),dataPC(:,2),dataPC(:,3),classLet,{'r','g','c','b'})
    str = sprintf('PCA on %s',signalname);
    title(str)
    xlabel('PC1'); ylabel('PC2'); zlabel('PC3')
    set(gca,'FontSize',14)
    
    hold on
    for k=1:4
        [~,centroid] = kmeans(dataPC((classNum==k),:),1);
        scatter(centroid(1),centroid(2))
        c = classLet(classNum==k);
        text(centroid(1),centroid(2),c(1),'FontSize',14);
    end
    hold off
    end
end



