function [cp,fscoring,featureSel] = classify(features,classNum,kfold,L,...
                                learner,option1,option2,option3,Fnum,k)
% classify Classification function
%   cp = classify(features,classNum,kfold,L,learner,option1,option2,...
%                 option3,Fnum,k) returns the cp classification 
%   performance object. features are the feature input data, with rows as 
%   observations and columns as features or dimensions. classNum is numeric 
%   vector with class labels for observations in feature. Using k-fold  
%   cross validation, data is divided in train and test sets set by kfold. 
%   Inside the cross validation loop, feature selection, oversampling and
%   classification are performed and the performance metrics are averaged 
%   over all the cross validation loops. This is again average by iterating
%   cross validation L times. learner is a string stating classifier to be
%   learned. option1, option2 and option3 are string/variable with
%   parameter tuning for corresponding learner. If learner have less than
%   three options, anything can put her, e.g. numeric 0. Fnum specifies
%   number of features used for training. If value is 0, all positive
%   weights features from ReliefF are used for training. k is the number of
%   nearest neighbors in the ReliefF feature ranking. 
%
%   [cp,fscoring] = classify(...) returns fscoring vector which are the
%   weight values for each feature from the ReliefF algorithm.
%
%   [cp,fscoring,featureSel] = classify(...) returns featureSel matrix with
%   all features from the feature selection process.
%

%   Created by Mathias Falk, last modified 01/09/16

  
    % initialize
    classtr = cellstr(char(classNum+'A'-1));         % Convert to string
    fscoring = 0;                                    % Feature scoring
    indices = crossvalind('Kfold',classtr,kfold);    % CV kfold indicies
    cp = classperf(classtr);        % classifier performance (cp) output
    
    
    for l=1:L   % L iterations of K-fold loop to average the kfold again
     
    % K-fold loop
    for i = 1:kfold
        
        % Train and test
        test = (indices == i); 
        train = ~test;

        
        % Feature Selection: ReliefF
        [ranked,weights] = relieff(features(train,:),classtr(train,:),k);
        featureSel = [];
        for j=1:length(find(weights>0))
            featureSel = [featureSel features(:,ranked(j))];
        end
        
        
        % Selected specific number of features, ranked by performance
        if Fnum > 0
            featureSel = featureSel(:,1:Fnum);
        end
        
        
        % Feature scoring 
        fscoring = fscoring + weights;
        
        
        
        % Oversampling by ADASYN?
        C = numel(unique(classNum));
        if C==2
            features_ADASYN_1 = ADASYN(featureSel(train,:),...
                                (classNum(train)-1), [], [], [], false);
            rows = size(features_ADASYN_1);
            features_ADASYN = [features_ADASYN_1;featureSel(train,:)];
            classNum_ADASYN = [ones(rows(1),1);classNum(train)];
            classLet_ADASYN = cellstr(char(classNum_ADASYN+'A'-1));
        else
            features_ADASYN = featureSel(train,:);
            classLet_ADASYN = classtr(train);
        end
   
    
        % Classification
        if strcmp(learner,'discr')
            obj = fitcdiscr(features_ADASYN,classLet_ADASYN,...
                            'DiscrimType',option1);        % build model
            class = predict(obj,featureSel(test,:));       % predict
            classperf(cp,class,test);                      % update cp
        elseif strcmp(learner,'dtree')
            obj = fitctree(features_ADASYN,classLet_ADASYN,...
                           'MaxNumSplits',option1);         
            class = predict(obj,featureSel(test,:));        
            classperf(cp,class,test);                     
        elseif strcmp(learner,'knn')
            obj = fitcknn(features_ADASYN,classLet_ADASYN,... 
                          'BreakTies','nearest','Standardize',true,...
                          'Distance',option1,'NumNeighbors',option2,...
                          'DistanceWeight',option3); 
            class = predict(obj,featureSel(test,:));               
            classperf(cp,class,test);                           
        elseif strcmp(learner,'svm')
            if numel(unique(classNum)) == 2
                obj = fitcsvm(features_ADASYN,classLet_ADASYN,...
                              'BoxConstraint',option3,...
                              'KernelFunction',option1,...
                              'KernelScale',option2,...
                              'Standardize',true); 
                class = predict(obj,featureSel(test,:));               
                classperf(cp,class,test);  
            else
                obj = fitcecoc(features_ADASYN,classLet_ADASYN); 
                class = predict(obj,featureSel(test,:));              
                classperf(cp,class,test);                           
            end
        else
            break
        end
        
    end
    
    end
    
end
    
    