% featProc Feature Processing
%   featProc does feature construction and uses CWT average coeffs in the 
%   CWTdata struct for all patients, all signals, and different periods. 
%   The freqeuncy band is divided into subbands and features mean, var, 
%   power, max peak, kurtosis, shewness are computed for each subband. 
%   Also, all power samples from CWT are structured according to signal, 
%   HRV, ABP and LDF. HRV is also normalized and reduced to an example with
%   only every 10th sample. PCAcalc function is called for each signal,
%   calculating PC scores and performing plotting for data visualization.
%

% Created by Mathias Falk, last modified 05/09/16


% initialize matrices
rangeData = [];
featureHyperSub = [];
 
for i=1:4 % 4 classes
    
    for j=1:10 % 10 patients
    
        if j == 4   % exclude patient 4 due to noise contamination
            continue
        end
        
        
        % Normalization, global max per patient per signal
        comMaxLen = min(cellfun(@numel,CWTdata3(j).HRV.CWTavg));
        CWTdata3(j).HRV.CWTavg = cellfun(@(x) x(1:comMaxLen),...
                    CWTdata3(j).HRV.CWTavg,'UniformOutput',false);
        patientHRVmax = cell2mat(CWTdata3(j).HRV.CWTavg);
        patientHRVmax = max(patientHRVmax(:));
        
        comMaxLen = min(cellfun(@numel,CWTdata(j).ABP.CWTavg));
        CWTdata(j).ABP.CWTavg = cellfun(@(x) x(1:comMaxLen),...
                    CWTdata(j).ABP.CWTavg,'UniformOutput',false);
        patientABPmax = cell2mat(CWTdata(j).ABP.CWTavg);
        patientABPmax = max(patientABPmax(:));
        
        comMaxLen = min(cellfun(@numel,CWTdata(j).LDF.CWTavg));
        CWTdata(j).LDF.CWTavg = cellfun(@(x) x(1:comMaxLen),...
                    CWTdata(j).LDF.CWTavg,'UniformOutput',false);
        patientLDFmax = cell2mat(CWTdata(j).LDF.CWTavg);
        patientLDFmax = max(patientLDFmax(:));
        
        
        % extract CWT data
        data = cell(3,2);
        data{1,1} = CWTdata(j).HRV.CWTavg{i};
        data{1,2} = CWTdata(j).HRV.CWTsig{i}.frequencies;
        data{2,1} = CWTdata(j).ABP.CWTavg{i};
        data{2,2} = CWTdata(j).ABP.CWTsig{i}.frequencies;
        data{3,1} = CWTdata(j).LDF.CWTavg{i};
        data{3,2} = CWTdata(j).LDF.CWTsig{i}.frequencies;
      
  
        featureHypoSub = [];
        
        for k=1:3 % 3
        
            % Divide in subbands
            range = cell(6,1);
            
            
            for l=1:352 % common max length CWTs
            
                if     (data{k,2}(l) >= Flim0) && (data{k,2}(l) < Flim1)
                    range{1} = [range{1} data{k,1}(l)];
                elseif (data{k,2}(l) >= Flim1) && (data{k,2}(l) < Flim2)
                    range{2} = [range{2} data{k,1}(l)];
                elseif (data{k,2}(l) >= Flim2) && (data{k,2}(l) < Flim3)
                    range{3} = [range{3} data{k,1}(l)];
                elseif (data{k,2}(l) >= Flim3) && (data{k,2}(l) < Flim4)
                    range{4} = [range{4} data{k,1}(l)];
                elseif (data{k,2}(l) >= Flim4) && (data{k,2}(l) < Flim5)
                    range{5} = [range{5} data{k,1}(l)];
                elseif (data{k,2}(l) >= Flim5) && (data{k,2}(l) < Flim6)
                    range{6} = [range{6} data{k,1}(l)];
                end

            end
      
            
            % Compute features
            featureSub = [];      
            for m=1:6 % 6 subbands
     
                f1 = mean(range{m});                        % mean
                f2 = var(range{m});                         % variance
                f3 = skewness(range{m});                    % skewness
                f4 = kurtosis(range{m});                    % kurtosis
                f5 = (1/length(range{m}))*sum(range{m}.^2); % power
                f6 = max(findpeaks(range{m}));              % max peak
                    if isempty(f6)                      % zero if no peak
                        f6 = 0;
                    end
                
                featureSub = [featureSub f1 f2 f3 f4 f5 f6];
            end
            
            featureHypoSub = [featureHypoSub featureSub];
            


            % Ceil number of samples in each subband for later concatenation
            range{1} = range{1}(1:39);
            range{2} = range{2}(1:34);
            range{3} = range{3}(1:50);
            range{4} = range{4}(1:42);
            range{5} = range{5}(1:45);
            range{6} = range{6}(1:64);
           
            rangeData = [rangeData; range'];
          
        end
        
        featureHyperSub = [featureHyperSub; featureHypoSub];
        
    end

end


% Orginize all data in matrices
rangeDataHRV = rangeData(1:3:end,:);
rangeDataABP = rangeData(2:3:end,:);
rangeDataLDF = rangeData(3:3:end,:);

dataHRV = cell2mat(rangeDataHRV);
dataABP = cell2mat(rangeDataABP);
dataLDF = cell2mat(rangeDataLDF);


% Normalization
dataHRV_stdnorm = [];
for i=1:length(dataHRV)
    dataHRV_stdnorm = [dataHRV_stdnorm dataHRV(:,i)./std(dataHRV(:,i))];
end

% Every 10th sample
dataHRV10 = dataHRV(:,1:10:end);


% PCA plotting and feature extraction
scoresHRV = PCAcalc(dataHRV,classNum4,'HRV');
scoresABP = PCAcalc(dataABP,classNum4,'ABP');
scoresLDF = PCAcalc(dataLDF,classNum4,'LDF');
scoresPCA = [scoresHRV scoresABP scoresLDF];

scoresHRV10 = PCAcalc(dataHRV10,classNum4,'HRV every 10th score');
scoresHRVnorm = PCAcalc(dataHRV_stdnorm,classNum4,'HRV normalized');

