function [anova,c,swstats] = stattest(x,classNum,alpha,signalname,hist)
% stattest Statistical tests
%   [anova] = stattest(x,d,alpha,signalname,hist) return variable 
%   anova which is the anova table with ch-sq statistic and probability of
%   rejecting the null hypothesis samples from same distribution, or 
%   comparison of the mean ranks. x is a 1xN vector with all observations 
%   from one feature. classNum is class label interger (>0) vector. K-W 
%   test is performed with the kruskalwallis function in the Statisics and 
%   Machine Learning Toolbox. The samples for each class are box plotted. 
%   alpha is the statistical significance level for performing subsequent 
%   Bonferroni correction as post-hoc to K-W test, seeing which classes 
%   have different means. signalname is a string input used for plotting. 
%   hist is a string, where 'on' plots histogram for observations from each
%   class.
%
%   [anova,c] = stattest(x,d,alpha,signalname,hist) returns c which 
%   is a matrix with metrics from Bonferroni correction multicomparison
%   test.
%
%   [anova,c,swstats] = stattest(x,d,alpha,signalname,hist) returns matrix
%   swstat with info from the swtest function accessed from MATLAB File
%   Exchange. 
%
%   Example: 
%   anova = stattest(featureHyperSub(:,1),classNum4,0.05,'HRV power band 1,'off')

%   Created by Mathias Falk, last modified 05/09/16


    classes = {};
    numClass = numel(unique(classNum)); 
    for i=1:numClass
        classes{i} = x(classNum==i);
    end
    
    
    % Histogram
    if strcmp(hist,'on')
        asciiValues = 1:numClass;
        asciiChars = char(asciiValues+'A'-1);
        for i=1:numClass%d
            figure; histogram(classes{i},10)
            str = sprintf('class %s',asciiChars(i));
            title(['Histogram of ',str,' ',signalname])
        end
    end
    
    
    % SW test
    swstats = [];
    for i=1:numClass
        [h,p,w] = swtest(classes{i},alpha);
        swstats = [swstats; h,p,w];
    end
    
    
    % KW test
    [~,anova,stats] = kruskalwallis(x,classNum,'off');
    title(['KW test ',signalname])
    boxplot(x,classNum)
    figure
    c = multcompare(stats,'alpha',alpha,'ctype','bonferroni');        
    title(['Bonferroni correction of KW test ',signalname])
end
    
    